/*
  Rodent, a UCI chess playing engine derived from Sungorus 1.4
  Copyright (C) 2009-2011 Pablo Vazquez (Sungorus author)
  Copyright (C) 2011-2012 Pawel Koziol

  Rodent is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published 
  by the Free Software Foundation, either version 3 of the License, 
  or (at your option) any later version.

  Rodent is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty 
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _SEARCHER_H
#define _SEARCHER_H

#define IS_PV 1
#define NO_PV 0
#define WAS_NULL 1
#define NO_NULL  0

struct sSearcher {
private:
	int flagAbortSearch;
	void CheckInput(void);
	int isReporting;

	int Blunder(sPosition *p, int ply, int depth, int flag, int move, int lastMove, int flagInCheck);
	
	// report.c
	void DisplayCurrmove(int move, int movesTried);
	void DisplayDepth(void);
	void DisplayPv(int score, int *pv);
	void DisplayRootInfo(void);
	void DisplaySettings(void);
	void DisplaySavedIterationTime();
	void DisplaySpeed(void);
	U32  GetNps(int nodes, int time);

	int DrawBy50Moves(sPosition *p);
	void Iterate(sPosition *p, int *);
	int IsRepetition(sPosition *p);
	int IsMoveOrdinary(int flagMoveType);
	int Search(sPosition *p, int ply, int alpha, int beta, int depth, int isPvNode, int wasNull, int lastMove, int *pv);
	int SetFutilityMargin(int depth);
	int SetLmrDepth(int move, int movesTried);
	int SetNullDepth(int depth);
	int Quiesce(sPosition *p, int ply, int qDepth, int alpha, int beta, int *pv);
	int CountLegalMoves(sPosition *p, int cap);
	int AdjustRootDepth(sPosition *p);
	int RecognizeDraw(sPosition *p);
public:
	void Think(sPosition *, int *);
	int VerifyValue(sPosition *p, int depth, int move);
};

extern struct sSearcher Searcher;

#endif